# lumi-g-test

Simple MPI-tool to check sanity of LUMI-G environment at runtime. Testcase is:
- MPI application with 8 MPI ranks (equal to number of GPUs on a node)
- OpenMP cycles: 7 OMP threads
- uses `hip` for GPU-related tests

We check that:
- build is OK
- environment setting and runtime configuration is OK
- bandwidth charachteristics on 2 nodes are OK (applicable also to any even number of nodes).

What we do at runtime:
- print CPU core affinity info for each rank, for each thread in OpenMP region
- print NIC affinity info for the first rank
- print GPU affinity info for each rank
- STREAM test on GPU (per rank, on all cards simultaneously)
- Bandwidth H2D/D2H test for GPUs (per rank, on all cards simultaneously, pinned and pageable)
- Bandwidth of simple MPI p2p exchange (per rank, on all ranks cards simultaneously)
    - using GPU memory
    - using CPU memory
    - unidirectional
    - bidirectional

We check the results to meet the hardcoded limits (see the check.inl file). When a check fails we print a message starting with `LUMI-G-TEST>>>>>`. It is printed even when the general output for this utility is switched off.

Build options can be given as make utility arguments:
- `OUTPUT=TRUE|FALSE` (default: `TRUE`) - one can switch off the tests output: utility will be silent of no errors
- `BUILD=Release|Debug` (default: `Release`) - optimizing/non-optimizing builds switch
- `CHECKS=TRUE|FALSE` (default: `TRUE`) - do the comparisons with hardcoded contol values in `check.inl`
- `TIMEOUT=0|N` (default: 0) - set the timeout for the whole execution in second (no timeout if 0)
- `MPI=TRUE|FALSE` (default: `TRUE`) - do MPI tests
- `GPU=TRUE|FALSE` (default: `TRUE`) - do GPU tests

The generated binary name depends on the GPU and MPI options compination, see the Makefile. Also consult the Makefile for a set of necessary modules on LUMI-G environment (must be loaded both in compile time and runtime.

The `psubmit.opt` file can be used for quick test on LUMI-G. That is a config file for a `psubmit` utility: `https://github.com/a-v-medvedev/psubmit`. Example:

```sh
psubmit.sh -n2 -a ./lumi-g-test-nogpu-mpi
```

