#include <stdlib.h>
#include <assert.h>
#include "alloc.h"
#include "defs.h"

namespace sys {

char *gpu_alloc(size_t size) {
#if WITH_GPU
    char *ptr;
    HIP_CALL(hipMalloc((void**)&ptr, size));
    HIP_CALL(hipMemset(ptr, 0, sizeof(size)));
    return ptr;
#else
    (void)size;
    assert(0);
    return nullptr;
#endif
}

void gpu_free(void *ptr) {
#if WITH_GPU
    HIP_CALL(hipFree(ptr));
#else
    (void)ptr;
#endif
}

char *cpu_alloc_pinned(size_t size) {
#if WITH_GPU
    char *ptr;
    HIP_CALL(hipHostMalloc(&ptr, size, 0));
    return ptr;
#else
    (void)size;
    assert(0);
    return nullptr;
#endif
}

void cpu_free_pinned(void *ptr) {
#if WITH_GPU
    HIP_CALL(hipHostFree(ptr));
#else
    (void)ptr;
#endif
}

char *cpu_alloc_pageable(size_t size) {
    char *ptr;
    ptr = (char *)calloc(size, 1);
    return ptr;
}

void cpu_free_pageable(void *ptr) {
    free(ptr);
}

}

