#pragma once

#include <iostream>
#include <sstream>
#include <iomanip>
#include <vector>
#include <string>
#include <unistd.h>

#ifdef __HIP__
#include "hip/hip_runtime.h"
#include "hip/hip_runtime_api.h"

static inline void HIP_CALL(hipError_t err) { 
    if (err != hipSuccess) { 
	    throw std::runtime_error(hipGetErrorString(err)); 
    } 
    auto err_last = hipGetLastError(); 
    if (err_last != hipSuccess) { 
	    throw std::runtime_error(hipGetErrorString(err_last)); 
    } 
}
#endif

#ifdef WITH_MPI
#define MPI_CALL(call) \
{ \
            int mpierr = call; \
            if (mpierr != 0) \
        { \
            char errstr[1024]; \
            int len = 0, rank = 0; \
            if (len >= 1024) \
            MPI_Error_string(mpierr, errstr, &len); \
            if (len >= 1024) \
                errstr[1023] = 0; \
                fprintf(stderr, "%s:%d rank=%i %s\n", __FILE__, \
                                                      __LINE__, MPI_Comm_rank(MPI_COMM_WORLD, &rank), errstr); \
                        throw std::runtime_error("MPI function call returned error state"); \
                } \
} 
#endif

static inline std::string rank2str(int rank) {
	return std::string("rank=") + std::to_string(rank) + ": ";
}

static inline void print(int rank, const std::string &str) {
    (void)rank;
    (void)str;
#ifdef WITH_OUTPUT
    std::cout << rank2str(rank) << str << std::endl;
#endif
}

static inline void printx(int rank, const std::string &str) {
    static char hostname[256] = {0};
    if (hostname[0] == 0) {
        (void)gethostname(hostname, 256);
    }
    std::cerr << "LUMI-G-TEST>>>>> " << hostname << " " << rank2str(rank) << str << std::endl;
    std::cout << "LUMI-G-TEST>>>>> " << hostname << " " << rank2str(rank) << str << std::endl;
}

using real = double;

struct oglobal {
    std::string header;
    std::vector<double> values;
};

struct olocal {
    std::string header;
    std::vector<std::vector<double>> values;
};

